@@ -49,6 +49,9 @@
 #' are requested they will be added sequentially outwards starting from the plot
 #' border. The axis will always be placed on the outside of the last scale.
 #'
+#' \code{axis_geo_phylo(...)} is shorthand for
+#' \code{axis_geo(..., phylo = TRUE)}.
+#'
 #' @param side \code{integer}. Which side to add the axis to (\code{1}: bottom,
 #'   the default; \code{2}: left; \code{3}: top; \code{4}: right).
 #' @param intervals The interval information to use to plot the axis: either A)
@@ -113,6 +116,12 @@
 #'   \code{at}, or b) a custom \code{character} or \code{expression} vector of
 #'   labels to be placed at the tick marks. If \code{at} is specified, this
 #'   argument is passed directly to \code{\link[graphics]{axis}}.
+#' @param phylo \code{logical}. Is the base plot a phylogeny generated by
+#'   \code{\link[ape]{plot.phylo}}, \code{\link[phytools]{plotTree}},
+#'   \code{\link[phytools]{plotSimmap}}, etc?
+#' @param root.time \code{numeric}. If \code{phylo} is \code{TRUE}, this is the
+#'   time assigned to the root node of the tree. By default, this is taken from
+#'   the \code{root.time} element of the plotted tree.
 #' @param ... Further arguments that are passed directly to
 #'   \code{\link[graphics]{axis}}.
 #'
@@ -126,6 +135,7 @@
 #' @importFrom graphics rect text clip axis par segments
 #' @importFrom deeptime getScaleData
 #' @importFrom methods is
+#' @importFrom ape .PlotPhyloEnv
 #' @examples
 #' # track user par
 #' oldpar <- par(no.readonly = TRUE)
@@ -182,6 +192,22 @@
 #' # the line argument here depends on the absolute size of the plot
 #' title(xlab = "Time (Ma)", line = 4)
 #'
+#' @examplesIf require(phytools)
+#' # scale with phylogeny
+#' library(phytools)
+#' data(mammal.tree)
+#' plot(mammal.tree)
+#' axis_geo_phylo()
+#' title(xlab = "Time (Ma)", line = 4)
+#'
+#' @examplesIf require(paleotree)
+#' # scale with fossil phylogeny
+#' library(paleotree)
+#' data(RaiaCopesRule)
+#' plot(ceratopsianTreeRaia)
+#' axis_geo_phylo()
+#' title(xlab = "Time (Ma)", line = 4)
+#'
 #' # reset user par
 #' par(oldpar)
 #' @export
@@ -198,11 +224,25 @@ axis_geo <- function(
     # applied to the entire axis:
     bkgd = "grey90", neg = FALSE, exact = FALSE, round = FALSE,
     # passed to axis():
-    tick_at = NULL, tick_labels = TRUE, ...
+    tick_at = NULL, tick_labels = TRUE, phylo = FALSE, root.time = NULL,
+    ...
     ) {
   intervals <- make_list(intervals)
   n_scales <- length(intervals)
 
+  if (phylo) {
+    # adapted from ape
+    lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
+    type <- lastPP$type
+    backward <- lastPP$direction %in% c("leftwards", "downwards")
+    if (type == "unrooted")
+      stop("axis_geo() not available for unrooted plots;
+           try ape::add.scale.bar()")
+    if (type %in% c("radial", "fan"))
+      stop("axis_geo() not meaningful for radial or fan plots")
+    if (is.null(root.time)) root.time <- lastPP$root.time
+  }
+
   height <- rep(make_list(height), length.out = n_scales)
   if (!all(sapply(height, function(x) is.numeric(x) && length(x) == 1))) {
     stop("Invalid value supplied for height, must be a single numeric value
@@ -269,11 +309,23 @@ axis_geo <- function(
 
   # height should be "negative" if the axis is reversed
   if (side %in% c(1, 3)) {
+    if (phylo) {
+      x_range <- range(lastPP$xx)
+      phylo_lims <- c(max(lastPP$x.lim[1], x_range[1]),
+                      min(lastPP$x.lim[2], x_range[2]))
+      clip_lims[1:2] <- plot_lims[1:2] <- phylo_lims
+    }
     height <- lapply(height, function(ht) {
       abs_ht <- ht * abs(plot_lims[3] - plot_lims[4])
       abs_ht * c(-1, 1)[(plot_lims[3] < plot_lims[4]) + 1]
     })
   } else if (side %in% c(2, 4)) {
+    if (phylo) {
+      y_range <- range(lastPP$yy)
+      phylo_lims <- c(max(lastPP$y.lim[1], y_range[1]),
+                      min(lastPP$y.lim[2], y_range[2]))
+      clip_lims[3:4] <- plot_lims[3:4] <- phylo_lims
+    }
     height <- lapply(height, function(ht) {
       abs_ht <- ht * abs(plot_lims[1] - plot_lims[2])
       abs_ht * c(-1, 1)[(plot_lims[1] < plot_lims[2]) + 1]
@@ -282,6 +334,20 @@ axis_geo <- function(
     stop("Invalid value supplied for side, must be 1, 2, 3, or 4",
          call. = FALSE)
   }
+  if (phylo) {
+    # adapted (and fixed) from ape
+    phylo_scale <- c(0, phylo_lims[2] - phylo_lims[1])
+    if (!backward) phylo_scale <- phylo_scale[2:1]
+    if (!is.null(root.time)) {
+      if (backward) {
+        phylo_scale <- phylo_scale + root.time - phylo_scale[2]
+      } else {
+        phylo_scale <- phylo_scale + root.time - phylo_lims[2]
+      }
+    }
+    phylo_beta <- diff(phylo_lims) / diff(phylo_scale)
+    phylo_alpha <- phylo_lims[1] - phylo_beta * phylo_scale[1]
+  }
 
   # expand clipping outside the desired axis
   height_sum <- do.call(sum, height)
@@ -361,6 +427,12 @@ axis_geo <- function(
       scale_intervals$max_age <- -1 * (scale_intervals$max_age)
       scale_intervals$min_age <- -1 * (scale_intervals$min_age)
     }
+    if (phylo) {
+      scale_intervals$max_age <- phylo_beta * scale_intervals$max_age +
+                                   phylo_alpha
+      scale_intervals$min_age <- phylo_beta * scale_intervals$min_age +
+                                   phylo_alpha
+    }
     scale_intervals$mid_age <-
       (scale_intervals$max_age + scale_intervals$min_age) / 2
 
@@ -499,6 +571,17 @@ axis_geo <- function(
     } else {
       tick_labels <- as.character(tick_at) # removes trailing zeros
     }
+  } else if (phylo) {
+    tick_labels <- pretty(phylo_scale)
+    tick_at <- phylo_beta * tick_labels + phylo_alpha
+    # Filter to only ticks within limits
+    if (side %in% c(1, 3)) {
+      tick_inc <- which(tick_at <= scale_lims[2] & tick_at >= scale_lims[1])
+    } else {
+      tick_inc <- which(tick_at <= scale_lims[4] & tick_at >= scale_lims[3])
+    }
+    tick_at <- tick_at[tick_inc]
+    tick_labels <- tick_labels[tick_inc]
   }
   if (side == 1) {
     axis(side = side, pos = clip_lims[3], at = tick_at, labels = tick_labels,
@@ -518,6 +601,12 @@ axis_geo <- function(
   # this tricky/messy...
 }
 
+#' @export
+#' @rdname axis_geo
+axis_geo_phylo <- function(...) {
+  axis_geo(..., phylo = TRUE)
+}
+
 make_list <- function(x) {
   if (is.list(x) && !is(x, "data.frame")) x else list(x)
 }
